"""Alpacastocks exchange subclass"""
from freqtrade.exchange import Exchange
import logging
from alpaca.trading.client import TradingClient
from alpaca.trading.requests import GetAssetsRequest
from alpaca.trading.enums import AssetClass
import pandas as pd
from freqtrade.constants import Config

logger = logging.getLogger(__name__)

class Alpacastocks(Exchange):
    """
    Alpacastocks exchange class. Contains adjustments needed for Freqtrade to work
    with this exchange.
    """

    DECIMAL_PLACES = 2
    SIGNIFICANT_DIGITS = 3
    TICK_SIZE = 4

    _ft_has_default = {
        "stoploss_on_exchange": False,
        "order_time_in_force": ["GTC"],
        "ohlcv_candle_limit": 500,
        "ohlcv_has_history": True,
        "ohlcv_partial_candle": True,
        "ohlcv_require_since": False,
        "ohlcv_volume_currency": "base",
        "tickers_have_quoteVolume": True,
        "tickers_have_percentage": True,
        "tickers_have_bid_ask": True,
        "tickers_have_price": True,
        "trades_limit": 1000,
        "trades_pagination": "time",
        "trades_pagination_arg": "since",
        "trades_has_history": False,
        "l2_limit_range": None,
        "l2_limit_range_required": True,
        "mark_ohlcv_price": "mark",
        "mark_ohlcv_timeframe": "8h",
        "funding_fee_timeframe": "8h",
        "ccxt_futures_name": "swap",
        "needs_trading_fees": False,
        "order_props_in_contracts": ["amount", "filled", "remaining"],
        "market_props_in_contracts": ["status"],
        "market_has_ticker": False,
        "market_has_ohlcv": True,
        "order_has_status": True,
        "order_has_type": True,
        "order_has_side": True,
        "order_has_time_in_force": False,
        "order_has_price": True,
        "order_has_amount": True,
        "order_has_cost": False,
        "order_has_fee": False,
        "order_has_slippage": False,
        "order_has_filled": True,
        "order_has_remaining": True,
        "order_has_status_history": False,
        "ws_enabled": False,
    }

    def __init__(self, config: dict, *, exchange_config: dict | None = None, validate: bool = True, load_leverage_tiers: bool = False) -> None:
        super().__init__(config, exchange_config=exchange_config, validate=validate, load_leverage_tiers=load_leverage_tiers)

        exchange_conf = exchange_config if exchange_config else config.get('exchange', {})

        # Retrieve API key and secret
        key = exchange_conf.get("key")
        secret = exchange_conf.get("secret")

        # Log confirmation of configuration receipt
        logger.info(f"Configuration data: Key: {key}, Secret: {secret}")

        # Log the entire config for debugging
        #logger.debug(f"Full configuration: {config}")

        # Determine base URL based on dry_run setting
        dry_run = config.get("dry_run", False)  # Default to False if dry_run is not specified
        logger.info(f"Dry run setting: {dry_run}")

        if dry_run:
            base_url = 'https://paper-api.alpaca.markets'
            logger.info(f"Connecting to Alpaca paper trading.")
        else:
            base_url = 'https://api.alpaca.markets'
            logger.info(f"Connecting to Alpaca live trading.")

        # Initialize TradingClient
        self.trading_client = TradingClient(key, secret, paper=dry_run)

        # Fetch assets
        search_params = GetAssetsRequest(asset_class=AssetClass.US_EQUITY)
        assets = self.trading_client.get_all_assets(search_params)
        assets_dict = [dict(item) for item in assets]

        # Convert assets to DataFrame
        df = pd.DataFrame.from_records(assets_dict)
        #logger.info(f"Assets DataFrame: {df.head()}")

    def get_markets(self, reload=False, params=None):
        try:
            if not hasattr(self, '_markets') or reload:
                search_params = GetAssetsRequest(asset_class=AssetClass.US_EQUITY)
                assets = self.trading_client.get_all_assets(search_params)
                assets_dict = [dict(item) for item in assets]
                self._markets = {}
                for asset in assets_dict:
                    pair = f"{asset['symbol']}/USD"
                    self._markets[pair] = {
                        'id': pair,
                        'symbol': pair,
                        'base': asset['symbol'],
                        'quote': 'USD',
                        'spot': True,
                        'margin': False,
                        'active': asset['status'] == 'active',
                        'maker': 0.001,
                        'taker': 0.002,
                        'info': asset,
                        'precision': {
                            'amount': 8,
                            'price': 8
                        },
                        'limits': {
                            'amount': {
                                'min': 0.001,
                                'max': 1000000
                            },
                            'price': {
                                'min': 0.01,
                                'max': 1000000
                            },
                            'cost': {
                                'min': 0.01,
                                'max': 1000000
                            },
                        },
                        'future': False,
                        'option': False,
                        'linear': True,
                        'inverse': False,
                        'contractSize': 1,
                        'expiry': None,
                        'expiry_date': None,
                        'strike': None,
                        'underlying': None,
                        'settle': None,
                        'settleDate': None,
                        'listing': None,
                        'listed': None,
                        'market_type': 'spot',
                    }
                    #logger.debug(f"Added pair: {pair}")

            #logger.info(f"Retrieved markets: {self._markets}")
            return self._markets
        except Exception as e:
            logger.error(f"Failed to retrieve markets: {str(e)}")
            return {}

    @property
    def name(self):
        return "alpacastocks"

    @property
    def markets(self):
        return self._markets

    def get_fee(self, symbol, now=None, taker_or_maker='maker'):
        # Replace with the actual fee calculation for the symbol
        fee = {
           'maker': 0.001,
            'taker': 0.002,
        }
        return fee.get(taker_or_maker, 0)

    @property
    def precisionMode(self):
        return self.DECIMAL_PLACES

    @property
    def precision_mode_price(self):
        return self.precisionMode

    def validate_required_startup_candles(self, startup_candle_count, timeframe):
        pass

    def get_precision_price(self, pair):
        precision = {
            'TSLA/USD': 8,
            'MSFT/USD': 8,
            'AAPL/USD': 8,
            # Add more pairs as needed
        }
        return precision.get(pair, self.precision_mode_price)

    def get_max_leverage(self, pair, stake_amount):
        return 1

    def get_min_pair_stake_amount(self, *args, **kwargs):
        return 1

    def get_max_pair_stake_amount(self, *args, **kwargs):
        return 1000000

    def get_pair_base_currency(self, pair):
        return pair.split('/')[0]

    def get_pair_quote_currency(self, pair):
        return pair.split('/')[1]

    def get_contract_size(self, pair):
        return 1

    def get_precision_amount(self, pair):
        return 8

    @property
    def margin_mode(self):
        return None

    def get_liquidation_price(self, pair, amount, current_price=None, order_side=None, order_type=None, open_rate=None, is_short=None, stake_amount=None, leverage=None, wallet_balance=None):
        # Implementation here
        return None

    def update_liquidation_prices(exchange, trade, row):
        try:
            liquidation_price = exchange.get_liquidation_price(
                pair=trade.pair,
                current_price=row['close'],
                order_side=trade.order_side,
                amount=trade.amount,
                order_type=None,
                open_rate=row['open'],
                is_short=None,
                stake_amount=None,
                leverage=None,
                wallet_balance=None
            )
        except Exception as e:
            # Handle the exception
            raise e
        if liquidation_price is not None:
            # Update the liquidation price
            trade.liquidation_price = liquidation_price
        else:
            # Do not update the liquidation price
            pass

    def market_is_tradable(self, market):
        """
        Check if the market is tradable.
        :param market: Market dictionary
        :return: True if the market is tradable, False otherwise
        """
        return market['active'] and market['spot']

