"""Oanda exchange subclass, oanda.py"""
from freqtrade.exchange import Exchange
import logging
from oandapyV20.endpoints.accounts import AccountInstruments
from oandapyV20 import API
from oandapyV20 import V20Error
import pandas as pd
from freqtrade.constants import Config
import os
import json

logger = logging.getLogger(__name__)

class Oanda(Exchange):
    """
    Oanda exchange class. Contains adjustments needed for Freqtrade to work
    with this exchange.
    """

    DECIMAL_PLACES = 2
    SIGNIFICANT_DIGITS = 3
    TICK_SIZE = 4

    _ft_has_default = {
        "stoploss_on_exchange": False,
        "order_time_in_force": ["GTC"],
        "ohlcv_candle_limit": 500,
        "ohlcv_has_history": True,
        "ohlcv_partial_candle": True,
        "ohlcv_require_since": False,
        "ohlcv_volume_currency": "base",
        "tickers_have_quoteVolume": True,
        "tickers_have_percentage": True,
        "tickers_have_bid_ask": True,
        "tickers_have_price": True,
        "trades_limit": 1000,
        "trades_pagination": "time",
        "trades_pagination_arg": "since",
        "trades_has_history": False,
        "l2_limit_range": None,
        "l2_limit_range_required": True,
        "mark_ohlcv_price": "mark",
        "mark_ohlcv_timeframe": "8h",
        "funding_fee_timeframe": "8h",
        "ccxt_futures_name": "forex",
        "needs_trading_fees": False,
        "order_props_in_contracts": ["amount", "filled", "remaining"],
        "market_props_in_contracts": ["status"],
        "market_has_ticker": False,
        "market_has_ohlcv": True,
        "order_has_status": True,
        "order_has_type": True,
        "order_has_side": True,
        "order_has_time_in_force": False,
        "order_has_price": True,
        "order_has_amount": True,
        "order_has_cost": False,
        "order_has_fee": False,
        "order_has_slippage": False,
        "order_has_filled": True,
        "order_has_remaining": True,
        "order_has_status_history": False,
        "ws_enabled": False,
    }


    def __init__(self, config: dict, *, exchange_config: dict | None = None, validate: bool = True, load_leverage_tiers: bool = False) -> None:
        super().__init__(config, exchange_config=exchange_config, validate=validate, load_leverage_tiers=load_leverage_tiers)

        exchange_conf = exchange_config if exchange_config else config.get('exchange', {})

        # Retrieve API key and secret
        account_id = exchange_conf.get("key")
        access_token = exchange_conf.get("secret")

        # Log confirmation of configuration receipt
        logger.info(f"Configuration data: Account ID: {account_id}, Access Token: {access_token}")

        # Log the entire config for debugging
        # logger.debug(f"Full configuration: {config}")

        # Initialize API
        self.api = API(environment="practice" if config.get("dry_run", False) else "live", access_token=access_token)
        self.account_id = account_id


    def get_markets(self):
        try:
            if not hasattr(self, '_markets') or True:
                self._markets = {}
                # Correct method to retrieve instruments
                request = AccountInstruments(accountID=self.account_id)
                response = self.api.request(request)
                instruments = response.get('instruments', [])

                for instrument in instruments:
                    pair = instrument['name']
                    converted_pair = convert_pair_format(pair)
                    self._markets[converted_pair] = {
                        'id': converted_pair,
                        'symbol': converted_pair,
                        'base': converted_pair.split('/')[0],
                        'quote': converted_pair.split('/')[1],
                        'spot': True,
                        'margin': False,
                        'active': True,
                        'maker': 0.001,
                        'taker': 0.002,
                        'info': instrument,
                        'precision': {
                            'amount': 5,
                            'price': 5
                        },
                        'limits': {
                            'amount': {
                                'min': 0.01,
                                'max': 1000000
                            },
                            'price': {
                                'min': 0.01,
                                'max': 1000000
                            },
                            'cost': {
                                'min': 0.01,
                                'max': 1000000
                            },
                        },
                        'future': False,
                        'option': False,
                        'linear': True,
                        'inverse': False,
                        'contractSize': 1,
                        'expiry': None,
                        'expiry_date': None,
                        'strike': None,
                        'underlying': None,
                        'settle': None,
                        'settleDate': None,
                        'listing': None,
                        'listed': None,
                        'market_type': 'spot',
                    }
                # Save the markets data to a file
                with open(file_path, 'w') as file:
                    json.dump(self._markets, file)
                logger.info(f"Retrieved markets: {self._markets}")
            return self._markets
        except Exception as e:
            logger.error(f"Failed to retrieve markets: {str(e)}, fx assets seldom change, using a local saved file.")
            #print(f"Current working directory: {os.getcwd()}")
            file_path = 'user_data/data/oanda/fxmarkets.json'
            if os.path.exists(file_path):
                with open(file_path, 'r') as file:
                    self._markets = json.load(file)
                return self._markets
            else:
                logger.error(f"Failed to load local file.")
                return {}


    @property
    def name(self):
        return "oanda"


    @property
    def markets(self):
        return self._markets

    def get_fee(self, symbol, now=None, taker_or_maker='maker'):
        # Replace with the actual fee calculation for the symbol
        fee = {
            'maker': 0.001,
            'taker': 0.002,
        }
        return fee.get(taker_or_maker, 0)

    @property
    def precisionMode(self):
        return self.DECIMAL_PLACES

    @property
    def precision_mode_price(self):
        return self.precisionMode

    def validate_required_startup_candles(self, startup_candle_count, timeframe):
        pass

    def get_precision_price(self, pair):
        precision = {
            'EUR/USD': 3,
            'USD/JPY': 3,
            'GBP/USD': 3,
            # Add more pairs as needed
        }
        return precision.get(pair, self.precision_mode_price)

    def get_max_leverage(self, pair, stake_amount):
        return 1

    def get_min_pair_stake_amount(self, *args, **kwargs):
        return 0.01

    def get_max_pair_stake_amount(self, *args, **kwargs):
        return 1000000

    def get_pair_base_currency(self, pair):
        return pair.split('/')[0]

    def get_pair_quote_currency(self, pair):
        return pair.split('/')[1]

    def get_contract_size(self, pair):
        return 1

    def get_precision_amount(self, pair):
        return 5

    @property
    def margin_mode(self):
        return "cross"

    def get_liquidation_price(self, pair, amount, current_price=None, order_side=None, order_type=None, open_rate=None, is_short=None, stake_amount=None, leverage=None, wallet_balance=None, open_trades=0):
        # Implementation here
        return None

    def market_is_tradable(self, market):
        """
        Check if the market is tradable.
        :param market: Market dictionary
        :return: True if the market is tradable, False otherwise
        """
        return market['active'] and market['spot']
